/********************************* Beschreibung: *********************************
 *  Das Hochregallager von Fischertechnik wird unabhängig vom PC gesteuert
 *  und bedient. Beides wird von einem - mit etwas Hardware erweiterten - ftDuino 
 *  erledigt. Infos für den Benutzer werden mittels OLED Display bereit-
 *  gestellt, Werte werden mit einer Matrixtatstatur eingegeben.
 * 
 * Hardware: 
 * Aufgebautes Hochregallager aus dem Baukasten Automation Robots.
 * Aufbau und Verdrahtung erfolgen wie in der Anleitung zu 'Automation
 * Robots' beschrieben, allerdings ohne die Endschalter I5 und I6.
 * 1 x ftDuino, 1 x ftExtender, 1 x ft-OLED,
 * 1 x PCF8574 IO Expansion Board, 1 x I2c 4x4 Matrix Array Keypad,.
 * 
 * Programmierung:
 * Das Programm ist eng angelehnt an das Beispiel 'HighLevelRack' von Till Harbaum
 * das in der ftDuino Programmierumgebung unter Datei>Beispiele>Ftduino>HighLevelRack
 * zu finden ist.
 * Die Programme 'Arm.h' und 'Arm.cpp' so wie 'Tower.h' und 'Tower.cpp' müssen sich im selben
 * Verzeichnis wie diese ino-Datei befinden, außer dem wurden aus den beiden cpp-Dateien
 * alle Serial.println() Befehle entfernt um Platz im RAM-Speicher zu sparen.
 * Weitere Änderung sind in 'Tower.h':  ROW_FIRST = CM2Y(8*1.5+0.1);
 *                                      LIFT_OFFSET = CM2Y(0.6);
 * Diese beiden Änderungen dienen der genaueren Positionierung.                                     
 * Details zur Programmierumgebung und Programmierung können der ftDuino
 * Betriebsanleitung https://harbaum.github.io/ftduino/www/de/ entnommen weden.                         
************************************************************************************/
#include <Ftduino.h>
#include "Tower.h"
#include "Arm.h"
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <SPI.h>
#include <Keypad_I2C.h>
#include <Keypad.h>         
#include <Wire.h>

#define I2CADDR_TAST 0x20 // I2C Adresse der Tastatur
#define I2CADDR_OLED 0x3C // I2C Adresse der Anzeige

#define OLED_RESET 4 
Adafruit_SSD1306 display(OLED_RESET);

// Instanzen der Objekte des Modells
Arm arm;
Tower tower;

// Variablen und Konstanten zur Menüsteuerung
bool ok = true;
char modus = 0; // Unterschieden wird zwischen Einlagern 'B' und Auslagern 'A'
byte pos = 0;   // Für die 6 Positionen des Lagers
byte mnuNr = 1; // Menü 1 für die Modusauswahl Menü 2 für die Auswahl des Lagerplatzes
int mnuMaxTexte =6; // Anzahl der Textsegmente in den chr-Buffer "mnuEinsTexte" und "mnuZweiTexte"
const char modVorgabe = 'A';    // Vorgabe für den Modus
const char posVorgabe = '1';    // Vorgabe für Position

// Buffer mit Strings für Meldungen und Menüs bereitstellen
const char* mldgTexte[]={"ftDuino - Hochregal", "Initialisierung ...", "STOPP: Fehler Arm",
  "STOPP: Fehler Turm", "AKTION: Auslagern","AKTION: Einlagern"};
const char* mnuEinsTexte[]={"Modus:", "", "A Einlagern", "B Auslagern", "# Weiter", "", "?> "};
const char* mnuZweiTexte[]={"Position:", "", "1 ... 6 Position", "# Starten", "* Abbrechen", "", "?> "};

// Tastaturwerte festlegen
const byte ROWS = 4;            //Anzahl Reihen
const byte COLS = 4;            //Anzahl Zeilen

char keys[ROWS][COLS] = {
  {'1','2','3','A'}, 
  {'4','5','6','B'},
  {'7','8','9','C'},
  {'*','0','#','D'}
	};
byte rowPins[ROWS] = {0, 1, 2, 3}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {4, 5, 6, 7}; //connect to the column pinouts of the keypad
	
Keypad_I2C i2cKeypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS, I2CADDR_TAST );

void setup(){
  // Die Folientastatur initialisieren
  i2cKeypad.begin( );          
  
  // Das Display vorbereiten...
  display.begin(SSD1306_SWITCHCAPVCC, I2CADDR_OLED);
  display.setTextColor(WHITE);
  display.setTextSize(1);   // Display Textgröße festlegen
  //... und die Erste Anzeige ausgeben
  mldgAusgeben(0);
  delay(4000);

  // Den ftDuino initialisieren und entspr. Meldung ausgeben
  ftduino.init();
  arm.init();
  tower.init();
  mldgAusgeben(1);

  if(!arm.moveHome())
  {
    mldgAusgeben(2); 
    ok = false;
    return; 
  }
  if(!tower.moveHome())
  {
    mldgAusgeben(3); 
    ok = false;
    return; 
  }
  // Das erste Menü mit der Modus-Standardvorgabe anzeigen
  mnuEinsAusgeben(modVorgabe);  // Meü anzeigen
  mnuNr = 1;
}
/***************************** ENDE setup *****************************/  
void loop()
{
  char tasteGelesen = i2cKeypad.getKey(); // Tastatur abfragen
  while(ok && (tasteGelesen!= NO_KEY))    // Kein Fehler und eine Taste betätigt
  {
    if (mnuNr == 1)                           
    {
      if (tasteGelesen == 'A' || tasteGelesen == 'B') 
      {
        modus = tasteGelesen;
        mnuEinsAusgeben(tasteGelesen);  // Menü 1 mit einegegebenem Wert aktuallisieren
      }
      else
      {
        if (tasteGelesen == '#')  // Eingabe bzw. Vorgabe wurde bestätigt
        {
          if (!modus)             // Die Vorgabe wurde übernommen...
          {
            modus = modVorgabe;   //... kein Wert vom Bediener eingegeben
          }
          mnuNr = 2;                      // Menü 2 Vorbereiten ...
          pos = 0;
          tasteGelesen = 0;
          mnuZweiAusgeben(posVorgabe);    // ... und mit Vorgabewert anzeigen  
        }
      }
    }
    if (mnuNr == 2)
    {
      int zahlGelesen = tasteGelesen - 48; // den ASCII-char in eine Zahl konvertieren
      if (zahlGelesen  > 0 && zahlGelesen < 7)
      {
        pos = zahlGelesen;
        mnuZweiAusgeben(tasteGelesen);  // Menü 2 mit einegegebenem Wert aktuallisieren
      }
      else
      {
        if (tasteGelesen == '*')        // Abbrechen wurde gewählt
        {
          mnuNr = 1;                    // Menü 1 vorbereiten ...
          modus = 0;
          tasteGelesen = 0;
          mnuEinsAusgeben(modVorgabe);  // ... und mit Vorgabewert anzeigen
        }
        if (tasteGelesen == '#')        // Starten wurde gewählt
        {
          if (!pos)                     // Die Vorgabe wurde übernommen...
          {
            pos = posVorgabe - 48;      //... kein Wert vom Bediener eingegeben
          }                             // -48 entspr. ASCII zu Zahl
          StartMotoren(modus, pos);     // Die Motoren mit gewählten Parametern Starten
          pos = 0;                      // alles auf Anfangswerte zurücksetzen ...
          modus = 0;
          mnuNr = 1;
          mnuEinsAusgeben(modVorgabe);  // ... und Menü 1 mit Standardvorgabe anzeigen  
        }
      } 
    }
    tasteGelesen = NO_KEY;
  }
}
/****************************** ENDE loop ******************************/

void StartMotoren(char modus, int pos)
// Den Ablauf ensprechend der Eingegebenen Parameter starten
{
  if (modus == 'A')
  {
    mldgAusgeben(4);
    auslagern(pos);
  }
  else
  {
    mldgAusgeben(5);
    einlagern(pos);
  }
}

void auslagern(int pos)
{
  tower.moveTo(pos, false);
  arm.moveOut();
  tower.moveTo(pos, true);
  arm.moveIn();
  tower.moveTo(0, true);
  arm.moveOut();
  tower.moveTo(0, false);
  arm.moveIn();
  tower.moveHome();  
}

void einlagern(int pos)
{
  arm.moveOut();
  tower.moveTo(0, true);
  arm.moveIn();
  tower.moveTo(pos, true);
  arm.moveOut();
  tower.moveTo(pos, false);
  arm.moveIn();
  tower.moveTo(0, false);
  tower.moveHome();  
}

void mnuEinsAusgeben(char txtwahl)
{
    display.clearDisplay();
    display.setCursor(0,0);
    for (int i = 0; i <= mnuMaxTexte; i++)
    {
      if (i < mnuMaxTexte)
      {
        display.println(mnuEinsTexte[i]);  
      }
      else display.print(mnuEinsTexte[i]);
    }
    display.print(txtwahl);
    display.display(); 
 }

 void mnuZweiAusgeben(char txtwahl)
 {
    display.clearDisplay();
    display.setCursor(0,0);
    for (int i = 0; i <= mnuMaxTexte; i++)
    {
      if (i < mnuMaxTexte)
      {
        display.println(mnuZweiTexte[i]);  
      }
      else display.print(mnuZweiTexte[i]);
    }
    display.print(txtwahl);
    display.display(); 
 }

 void mldgAusgeben(int nr){
    display.clearDisplay();
    display.setCursor(0,25);
    display.println(mldgTexte[nr]);
    display.display(); 
 }
 
